
#include <stdio.h>
#include <signal.h>

#include "lint.h"
#include "interpret.h"
#include "object.h"
#include "unix_proc.h"

struct unix_proc unix_procs[MAX_UNIX_PROC];

void init_unix_procs()
{
  int i;
  
  for(i = 0; i < MAX_UNIX_PROC; i++)
  {
    unix_procs[i].pid = -1;
    unix_procs[i].state = -1;
    unix_procs[i].in_fd = -1;
    unix_procs[i].out_fd = -1;
    unix_procs[i].owner = NULL;
    unix_procs[i].read_handler = NULL;
    unix_procs[i].close_handler = NULL;
  }
}

int unix_exec(command, read_handler,  close_handler)
     char *command;
     char *read_handler;
     char *close_handler;
{
  struct svalue *ret;
  int    pipe1[2];
  int    pipe2[2];
  int    pid;
  int    fd;

  push_object(current_object);
  ret = apply_master_ob("valid_unix_exec", 1);
  if(!ret || ret->type != T_NUMBER ||
     ret->u.number == 0)
    return -1;

  for(fd = 0; fd < MAX_UNIX_PROC; fd++)
  {
    if(unix_procs[fd].pid != -1)
      continue;
    
    if(pipe(pipe1) < 0)
    {
       perror("pipe");
       return -1;
    }
    if(pipe(pipe2) < 0)
    {
       close(pipe1[0]);
       close(pipe1[1]);
       perror("pipe");
       return -1;
    }
    if((pid = fork()) == -1)
    {
       close(pipe1[0]);
       close(pipe1[1]);
       close(pipe2[0]);
       close(pipe2[1]);
       perror("fork");
       return -1;
    }
    /*
     * Child process. 
     * Dup pipe1 to stdin and pipe2 to stdout.
     * Close writeend of pipe1 and readend of pipe2
     * Exec the new process.
     */
    if(pid == 0)
    {
      char  *arglist[17];
      char  *argstring;
      char  *p;
      int    argc = 0;
      int    i;
  
      dup2(pipe1[0], 0);  
      dup2(pipe2[1], 1);  

      close(pipe1[1]);
      close(pipe2[0]);

      argstring = (char *) xalloc(strlen(command) + 1);
      strcpy(argstring, command);
      arglist[0] = (char *) argstring;
      for(p = (char *)argstring; *p; p++) 
      {
	if(*p == ' ')
	{
	  *p = '\0';
	  argc++;
	  arglist[argc] = p+1;
	}
      }
      arglist[++argc] = (char *) 0;
      execvp(arglist[0], arglist);
      perror("exec");
      return -1;
    }
    if(pid)
    {
      unix_procs[fd].pid     = pid;
      unix_procs[fd].in_fd   = pipe2[0];
      unix_procs[fd].out_fd  = pipe1[1];
      unix_procs[fd].owner  = current_object;
      add_ref(current_object, "unix_exec");
      
      unix_procs[fd].command = (char *) xalloc(strlen(command) + 1);
      strcpy(unix_procs[fd].command, command);
      
      if(read_handler)
      {
	unix_procs[fd].read_handler  = xalloc(strlen(read_handler) + 1);
	strcpy(unix_procs[fd].read_handler, read_handler);
      }
      if(close_handler)
      {
	unix_procs[fd].close_handler  = xalloc(strlen(close_handler) + 1);
	strcpy(unix_procs[fd].close_handler, close_handler);
      }      
      close(pipe1[0]);
      close(pipe2[1]);
    }
    return fd;
  }
  return -1;
}

void unix_close(int fd)
{
   close(unix_procs[fd].in_fd);
   close(unix_procs[fd].out_fd);
   unix_procs[fd].pid = -1;
   unix_procs[fd].state = -1;
   unix_procs[fd].in_fd = -1;
   unix_procs[fd].out_fd = -1;
   if(unix_procs[fd].read_handler)
      free(unix_procs[fd].read_handler);
   if(unix_procs[fd].close_handler)
      free(unix_procs[fd].close_handler);
   if(unix_procs[fd].owner)
      free_object(unix_procs[fd].owner, "unix_close");
}

int unix_write(fd, message)
     int fd;
     char *message;
{
  struct svalue *ret;
  int num;
  
  push_object(current_object);
  ret = apply_master_ob("valid_unix_exec", 1);
  if(!ret || ret->type != T_NUMBER ||
     ret->u.number == 0)
    return -1;

  if(unix_procs[fd].pid == -1)
    return -1;

  num = write(unix_procs[fd].out_fd, message, strlen(message));
  if(num < 0) 
  {
    return -1;
  }
  if(num == 0) 
  {
    unix_close(fd);
    return -1;
  }
  return num;
}
  
void handle_unix_read(fd) 
     int fd;
{
  char buf[BUFSIZE];
  int  ret;
  
  if((ret = read(unix_procs[fd].in_fd, buf, sizeof(buf) - 1)) < 0)
  {
    return;
  }
  if(ret == 0) 
  {
    unix_close(fd);
  }
  if(unix_procs[fd].owner->flags & O_DESTRUCTED)
  {
    unix_close(fd);
    return;
  }
  buf[ret] = '\0';
  push_number(fd);
  push_string(buf, STRING_MALLOC);
  apply(unix_procs[fd].read_handler, unix_procs[fd].owner, 2);
}

void handle_unix_write(fd)
{
}
